# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "ServedAssetFieldTypeEnum",
    },
)


class ServedAssetFieldTypeEnum(proto.Message):
    r"""Container for enum describing possible asset field types."""

    class ServedAssetFieldType(proto.Enum):
        r"""The possible asset field types.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            HEADLINE_1 (2):
                The asset is used in headline 1.
            HEADLINE_2 (3):
                The asset is used in headline 2.
            HEADLINE_3 (4):
                The asset is used in headline 3.
            DESCRIPTION_1 (5):
                The asset is used in description 1.
            DESCRIPTION_2 (6):
                The asset is used in description 2.
            HEADLINE (7):
                The asset was used in a headline. Use this only if there is
                only one headline in the ad. Otherwise, use the HEADLINE_1,
                HEADLINE_2 or HEADLINE_3 enums
            HEADLINE_IN_PORTRAIT (8):
                The asset was used as a headline in portrait
                image.
            LONG_HEADLINE (9):
                The asset was used in a long headline (used
                in MultiAssetResponsiveAd).
            DESCRIPTION (10):
                The asset was used in a description. Use this only if there
                is only one description in the ad. Otherwise, use the
                DESCRIPTION_1 or DESCRIPTION_@ enums
            DESCRIPTION_IN_PORTRAIT (11):
                The asset was used as description in portrait
                image.
            BUSINESS_NAME_IN_PORTRAIT (12):
                The asset was used as business name in
                portrait image.
            BUSINESS_NAME (13):
                The asset was used as business name.
            MARKETING_IMAGE (14):
                The asset was used as a marketing image.
            MARKETING_IMAGE_IN_PORTRAIT (15):
                The asset was used as a marketing image in
                portrait image.
            SQUARE_MARKETING_IMAGE (16):
                The asset was used as a square marketing
                image.
            PORTRAIT_MARKETING_IMAGE (17):
                The asset was used as a portrait marketing
                image.
            LOGO (18):
                The asset was used as a logo.
            LANDSCAPE_LOGO (19):
                The asset was used as a landscape logo.
            CALL_TO_ACTION (20):
                The asset was used as a call-to-action.
            YOU_TUBE_VIDEO (21):
                The asset was used as a YouTube video.
            SITELINK (22):
                This asset is used as a sitelink.
            CALL (23):
                This asset is used as a call.
            MOBILE_APP (24):
                This asset is used as a mobile app.
            CALLOUT (25):
                This asset is used as a callout.
            STRUCTURED_SNIPPET (26):
                This asset is used as a structured snippet.
            PRICE (27):
                This asset is used as a price.
            PROMOTION (28):
                This asset is used as a promotion.
            AD_IMAGE (29):
                This asset is used as an image.
            LEAD_FORM (30):
                The asset is used as a lead form.
            BUSINESS_LOGO (31):
                The asset is used as a business logo.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        HEADLINE_1 = 2
        HEADLINE_2 = 3
        HEADLINE_3 = 4
        DESCRIPTION_1 = 5
        DESCRIPTION_2 = 6
        HEADLINE = 7
        HEADLINE_IN_PORTRAIT = 8
        LONG_HEADLINE = 9
        DESCRIPTION = 10
        DESCRIPTION_IN_PORTRAIT = 11
        BUSINESS_NAME_IN_PORTRAIT = 12
        BUSINESS_NAME = 13
        MARKETING_IMAGE = 14
        MARKETING_IMAGE_IN_PORTRAIT = 15
        SQUARE_MARKETING_IMAGE = 16
        PORTRAIT_MARKETING_IMAGE = 17
        LOGO = 18
        LANDSCAPE_LOGO = 19
        CALL_TO_ACTION = 20
        YOU_TUBE_VIDEO = 21
        SITELINK = 22
        CALL = 23
        MOBILE_APP = 24
        CALLOUT = 25
        STRUCTURED_SNIPPET = 26
        PRICE = 27
        PROMOTION = 28
        AD_IMAGE = 29
        LEAD_FORM = 30
        BUSINESS_LOGO = 31


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import policy
from google.ads.googleads.v19.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v19.resources.types import ad as gagr_ad
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "MutateAdsRequest",
        "AdOperation",
        "MutateAdsResponse",
        "MutateAdResult",
    },
)


class MutateAdsRequest(proto.Message):
    r"""Request message for
    [AdService.MutateAds][google.ads.googleads.v19.services.AdService.MutateAds].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose ads
            are being modified.
        operations (MutableSequence[google.ads.googleads.v19.services.types.AdOperation]):
            Required. The list of operations to perform
            on individual ads.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        response_content_type (google.ads.googleads.v19.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["AdOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="AdOperation",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class AdOperation(proto.Message):
    r"""A single update operation on an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        policy_validation_parameter (google.ads.googleads.v19.common.types.PolicyValidationParameter):
            Configuration for how policies are validated.
        update (google.ads.googleads.v19.resources.types.Ad):
            Update operation: The ad is expected to have a valid
            resource name in this format:

            ``customers/{customer_id}/ads/{ad_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    policy_validation_parameter: policy.PolicyValidationParameter = proto.Field(
        proto.MESSAGE,
        number=3,
        message=policy.PolicyValidationParameter,
    )
    update: gagr_ad.Ad = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_ad.Ad,
    )


class MutateAdsResponse(proto.Message):
    r"""Response message for an ad mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v19.services.types.MutateAdResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateAdResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MutateAdResult",
    )


class MutateAdResult(proto.Message):
    r"""The result for the ad mutate.

    Attributes:
        resource_name (str):
            The resource name returned for successful
            operations.
        ad (google.ads.googleads.v19.resources.types.Ad):
            The mutated ad with only mutable fields after mutate. The
            field will only be returned when response_content_type is
            set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad: gagr_ad.Ad = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_ad.Ad,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AiMaxSearchTermAdCombinationView",
    },
)


class AiMaxSearchTermAdCombinationView(proto.Message):
    r"""AiMaxSearchTermAdCombinationView Resource.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the AI Max Search Term Ad
            Combination view AI Max Search Term Ad Combination view
            resource names have the form:

            ``customers/{customer_id}/aiMaxSearchTermAdCombinationViews/{ad_group_id}~{URL-base64_search_term}~{URL-base64_landing_page}~{URL-base64_headline}``
        ad_group (str):
            Output only. Ad group where the search term
            served.

            This field is a member of `oneof`_ ``_ad_group``.
        search_term (str):
            Output only. The search term that triggered
            the ad. This field is read-only.

            This field is a member of `oneof`_ ``_search_term``.
        landing_page (str):
            Output only. The destination URL, which was
            dynamically generated. This field is read-only.

            This field is a member of `oneof`_ ``_landing_page``.
        headline (str):
            Output only. The concatenated string containing headline
            assets for the ad. Up to three headline assets are
            concatenated, separated by " \| ". This field is read-only.

            This field is a member of `oneof`_ ``_headline``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    search_term: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    landing_page: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    headline: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "AutomaticallyCreatedAssetRemovalErrorEnum",
    },
)


class AutomaticallyCreatedAssetRemovalErrorEnum(proto.Message):
    r"""Container for enum describing possible automatically created
    asset removal errors.

    """

    class AutomaticallyCreatedAssetRemovalError(proto.Enum):
        r"""Enum describing possible automatically created asset removal
        errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            AD_DOES_NOT_EXIST (2):
                The ad does not exist.
            INVALID_AD_TYPE (3):
                Ad type is not supported. Only Responsive
                Search Ad type is supported.
            ASSET_DOES_NOT_EXIST (4):
                The asset does not exist.
            ASSET_FIELD_TYPE_DOES_NOT_MATCH (5):
                The asset field type does not match.
            NOT_AN_AUTOMATICALLY_CREATED_ASSET (6):
                Not an automatically created asset.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AD_DOES_NOT_EXIST = 2
        INVALID_AD_TYPE = 3
        ASSET_DOES_NOT_EXIST = 4
        ASSET_FIELD_TYPE_DOES_NOT_MATCH = 5
        NOT_AN_AUTOMATICALLY_CREATED_ASSET = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "UserListCustomerTypeCategoryEnum",
    },
)


class UserListCustomerTypeCategoryEnum(proto.Message):
    r"""The user list customer type categories."""

    class UserListCustomerTypeCategory(proto.Enum):
        r"""Enum containing possible user list customer type categories.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Unknown type.
            ALL_CUSTOMERS (2):
                Customer type category for all customers.
            PURCHASERS (3):
                Customer type category for all purchasers.
            HIGH_VALUE_CUSTOMERS (4):
                Customer type category for high value
                purchasers.
            DISENGAGED_CUSTOMERS (5):
                Customer type category for disengaged
                purchasers.
            QUALIFIED_LEADS (6):
                Customer type category for qualified leads.
            CONVERTED_LEADS (7):
                Customer type category for converted leads.
            PAID_SUBSCRIBERS (8):
                Customer type category for paid subscribers.
            LOYALTY_SIGN_UPS (9):
                Customer type category for loyalty signups.
            CART_ABANDONERS (10):
                Customer type category for cart abandoners.
            LOYALTY_TIER_1_MEMBERS (11):
                Customer type category for loyalty tier 1
                members.
            LOYALTY_TIER_2_MEMBERS (12):
                Customer type category for loyalty tier 2
                members.
            LOYALTY_TIER_3_MEMBERS (13):
                Customer type category for loyalty tier 3
                members.
            LOYALTY_TIER_4_MEMBERS (14):
                Customer type category for loyalty tier 4
                members.
            LOYALTY_TIER_5_MEMBERS (15):
                Customer type category for loyalty tier 5
                members.
            LOYALTY_TIER_6_MEMBERS (16):
                Customer type category for loyalty tier 6
                members.
            LOYALTY_TIER_7_MEMBERS (17):
                Customer type category for loyalty tier 7
                members.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ALL_CUSTOMERS = 2
        PURCHASERS = 3
        HIGH_VALUE_CUSTOMERS = 4
        DISENGAGED_CUSTOMERS = 5
        QUALIFIED_LEADS = 6
        CONVERTED_LEADS = 7
        PAID_SUBSCRIBERS = 8
        LOYALTY_SIGN_UPS = 9
        CART_ABANDONERS = 10
        LOYALTY_TIER_1_MEMBERS = 11
        LOYALTY_TIER_2_MEMBERS = 12
        LOYALTY_TIER_3_MEMBERS = 13
        LOYALTY_TIER_4_MEMBERS = 14
        LOYALTY_TIER_5_MEMBERS = 15
        LOYALTY_TIER_6_MEMBERS = 16
        LOYALTY_TIER_7_MEMBERS = 17


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "FeedItemValidationErrorEnum",
    },
)


class FeedItemValidationErrorEnum(proto.Message):
    r"""Container for enum describing possible validation errors of a
    feed item.

    """

    class FeedItemValidationError(proto.Enum):
        r"""The possible validation errors of a feed item.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            STRING_TOO_SHORT (2):
                String is too short.
            STRING_TOO_LONG (3):
                String is too long.
            VALUE_NOT_SPECIFIED (4):
                Value is not provided.
            INVALID_DOMESTIC_PHONE_NUMBER_FORMAT (5):
                Phone number format is invalid for region.
            INVALID_PHONE_NUMBER (6):
                String does not represent a phone number.
            PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY (7):
                Phone number format is not compatible with
                country code.
            PREMIUM_RATE_NUMBER_NOT_ALLOWED (8):
                Premium rate number is not allowed.
            DISALLOWED_NUMBER_TYPE (9):
                Phone number type is not allowed.
            VALUE_OUT_OF_RANGE (10):
                Specified value is outside of the valid
                range.
            CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY (11):
                Call tracking is not supported in the
                selected country.
            CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING (99):
                Customer is not on the allow-list for call
                tracking.
            INVALID_COUNTRY_CODE (13):
                Country code is invalid.
            INVALID_APP_ID (14):
                The specified mobile app id is invalid.
            MISSING_ATTRIBUTES_FOR_FIELDS (15):
                Some required field attributes are missing.
            INVALID_TYPE_ID (16):
                Invalid email button type for email
                extension.
            INVALID_EMAIL_ADDRESS (17):
                Email address is invalid.
            INVALID_HTTPS_URL (18):
                The HTTPS URL in email extension is invalid.
            MISSING_DELIVERY_ADDRESS (19):
                Delivery address is missing from email
                extension.
            START_DATE_AFTER_END_DATE (20):
                FeedItem scheduling start date comes after
                end date.
            MISSING_FEED_ITEM_START_TIME (21):
                FeedItem scheduling start time is missing.
            MISSING_FEED_ITEM_END_TIME (22):
                FeedItem scheduling end time is missing.
            MISSING_FEED_ITEM_ID (23):
                Cannot compute system attributes on a
                FeedItem that has no FeedItemId.
            VANITY_PHONE_NUMBER_NOT_ALLOWED (24):
                Call extension vanity phone numbers are not
                supported.
            INVALID_REVIEW_EXTENSION_SNIPPET (25):
                Invalid review text.
            INVALID_NUMBER_FORMAT (26):
                Invalid format for numeric value in ad
                parameter.
            INVALID_DATE_FORMAT (27):
                Invalid format for date value in ad
                parameter.
            INVALID_PRICE_FORMAT (28):
                Invalid format for price value in ad
                parameter.
            UNKNOWN_PLACEHOLDER_FIELD (29):
                Unrecognized type given for value in ad
                parameter.
            MISSING_ENHANCED_SITELINK_DESCRIPTION_LINE (30):
                Enhanced sitelinks must have both description
                lines specified.
            REVIEW_EXTENSION_SOURCE_INELIGIBLE (31):
                Review source is ineligible.
            HYPHENS_IN_REVIEW_EXTENSION_SNIPPET (32):
                Review text cannot contain hyphens or dashes.
            DOUBLE_QUOTES_IN_REVIEW_EXTENSION_SNIPPET (33):
                Review text cannot contain double quote
                characters.
            QUOTES_IN_REVIEW_EXTENSION_SNIPPET (34):
                Review text cannot contain quote characters.
            INVALID_FORM_ENCODED_PARAMS (35):
                Parameters are encoded in the wrong format.
            INVALID_URL_PARAMETER_NAME (36):
                URL parameter name must contain only letters,
                numbers, underscores, and dashes.
            NO_GEOCODING_RESULT (37):
                Cannot find address location.
            SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT (38):
                Review extension text has source name.
            CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED (39):
                Some phone numbers can be shorter than usual.
                Some of these short numbers are
                carrier-specific, and we disallow those in ad
                extensions because they will not be available to
                all users.
            INVALID_PLACEHOLDER_FIELD_ID (40):
                Triggered when a request references a
                placeholder field id that does not exist.
            INVALID_URL_TAG (41):
                URL contains invalid ValueTrack tags or
                format.
            LIST_TOO_LONG (42):
                Provided list exceeds acceptable size.
            INVALID_ATTRIBUTES_COMBINATION (43):
                Certain combinations of attributes aren't
                allowed to be specified in the same feed item.
            DUPLICATE_VALUES (44):
                An attribute has the same value repeatedly.
            INVALID_CALL_CONVERSION_ACTION_ID (45):
                Advertisers can link a conversion action with
                a phone number to indicate that sufficiently
                long calls forwarded to that phone number should
                be counted as conversions of the specified type.
                This is an error message indicating that the
                conversion action specified is invalid (for
                example, the conversion action does not exist
                within the appropriate Google Ads account, or it
                is a type of conversion not appropriate to phone
                call conversions).
            CANNOT_SET_WITHOUT_FINAL_URLS (46):
                Tracking template requires final url to be
                set.
            APP_ID_DOESNT_EXIST_IN_APP_STORE (47):
                An app id was provided that doesn't exist in
                the given app store.
            INVALID_FINAL_URL (48):
                Invalid U2 final url.
            INVALID_TRACKING_URL (49):
                Invalid U2 tracking url.
            INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL (50):
                Final URL should start from App download URL.
            LIST_TOO_SHORT (51):
                List provided is too short.
            INVALID_USER_ACTION (52):
                User Action field has invalid value.
            INVALID_TYPE_NAME (53):
                Type field has invalid value.
            INVALID_EVENT_CHANGE_STATUS (54):
                Change status for event is invalid.
            INVALID_SNIPPETS_HEADER (55):
                The header of a structured snippets extension
                is not one of the valid headers.
            INVALID_ANDROID_APP_LINK (56):
                Android app link is not formatted correctly
            NUMBER_TYPE_WITH_CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY (57):
                Phone number incompatible with call tracking
                for country.
            RESERVED_KEYWORD_OTHER (58):
                The input is identical to a reserved keyword
            DUPLICATE_OPTION_LABELS (59):
                Each option label in the message extension
                must be unique.
            DUPLICATE_OPTION_PREFILLS (60):
                Each option prefill in the message extension
                must be unique.
            UNEQUAL_LIST_LENGTHS (61):
                In message extensions, the number of optional
                labels and optional prefills must be the same.
            INCONSISTENT_CURRENCY_CODES (62):
                All currency codes in an ad extension must be
                the same.
            PRICE_EXTENSION_HAS_DUPLICATED_HEADERS (63):
                Headers in price extension are not unique.
            ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION (64):
                Header and description in an item are the
                same.
            PRICE_EXTENSION_HAS_TOO_FEW_ITEMS (65):
                Price extension has too few items.
            UNSUPPORTED_VALUE (66):
                The given value is not supported.
            INVALID_FINAL_MOBILE_URL (67):
                Invalid final mobile url.
            INVALID_KEYWORDLESS_AD_RULE_LABEL (68):
                The given string value of Label contains
                invalid characters
            VALUE_TRACK_PARAMETER_NOT_SUPPORTED (69):
                The given URL contains value track
                parameters.
            UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE (70):
                The given value is not supported in the
                selected language of an extension.
            INVALID_IOS_APP_LINK (71):
                The iOS app link is not formatted correctly.
            MISSING_IOS_APP_LINK_OR_IOS_APP_STORE_ID (72):
                iOS app link or iOS app store id is missing.
            PROMOTION_INVALID_TIME (73):
                Promotion time is invalid.
            PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF (74):
                Both the percent off and money amount off
                fields are set.
            PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT (75):
                Both the promotion code and orders over
                amount fields are set.
            TOO_MANY_DECIMAL_PLACES_SPECIFIED (76):
                Too many decimal places are specified.
            AD_CUSTOMIZERS_NOT_ALLOWED (77):
                Ad Customizers are present and not allowed.
            INVALID_LANGUAGE_CODE (78):
                Language code is not valid.
            UNSUPPORTED_LANGUAGE (79):
                Language is not supported.
            IF_FUNCTION_NOT_ALLOWED (80):
                IF Function is present and not allowed.
            INVALID_FINAL_URL_SUFFIX (81):
                Final url suffix is not valid.
            INVALID_TAG_IN_FINAL_URL_SUFFIX (82):
                Final url suffix contains an invalid tag.
            INVALID_FINAL_URL_SUFFIX_FORMAT (83):
                Final url suffix is formatted incorrectly.
            CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED (84):
                Consent for call recording, which is required
                for the use of call extensions, was not provided
                by the advertiser. See
                https://support.google.com/google-ads/answer/7412639.
            ONLY_ONE_DELIVERY_OPTION_IS_ALLOWED (85):
                Multiple message delivery options are set.
            NO_DELIVERY_OPTION_IS_SET (86):
                No message delivery option is set.
            INVALID_CONVERSION_REPORTING_STATE (87):
                String value of conversion reporting state
                field is not valid.
            IMAGE_SIZE_WRONG (88):
                Image size is not right.
            EMAIL_DELIVERY_NOT_AVAILABLE_IN_COUNTRY (89):
                Email delivery is not supported in the
                country specified in the country code field.
            AUTO_REPLY_NOT_AVAILABLE_IN_COUNTRY (90):
                Auto reply is not supported in the country
                specified in the country code field.
            INVALID_LATITUDE_VALUE (91):
                Invalid value specified for latitude.
            INVALID_LONGITUDE_VALUE (92):
                Invalid value specified for longitude.
            TOO_MANY_LABELS (93):
                Too many label fields provided.
            INVALID_IMAGE_URL (94):
                Invalid image url.
            MISSING_LATITUDE_VALUE (95):
                Latitude value is missing.
            MISSING_LONGITUDE_VALUE (96):
                Longitude value is missing.
            ADDRESS_NOT_FOUND (97):
                Unable to find address.
            ADDRESS_NOT_TARGETABLE (98):
                Cannot target provided address.
            INVALID_ASSET_ID (100):
                The specified asset ID does not exist.
            INCOMPATIBLE_ASSET_TYPE (101):
                The asset type cannot be set for the field.
            IMAGE_ERROR_UNEXPECTED_SIZE (102):
                The image has unexpected size.
            IMAGE_ERROR_ASPECT_RATIO_NOT_ALLOWED (103):
                The image aspect ratio is not allowed.
            IMAGE_ERROR_FILE_TOO_LARGE (104):
                The image file is too large.
            IMAGE_ERROR_FORMAT_NOT_ALLOWED (105):
                The image format is unsupported.
            IMAGE_ERROR_CONSTRAINTS_VIOLATED (106):
                Image violates constraints without more
                details.
            IMAGE_ERROR_SERVER_ERROR (107):
                An error occurred when validating image.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        STRING_TOO_SHORT = 2
        STRING_TOO_LONG = 3
        VALUE_NOT_SPECIFIED = 4
        INVALID_DOMESTIC_PHONE_NUMBER_FORMAT = 5
        INVALID_PHONE_NUMBER = 6
        PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY = 7
        PREMIUM_RATE_NUMBER_NOT_ALLOWED = 8
        DISALLOWED_NUMBER_TYPE = 9
        VALUE_OUT_OF_RANGE = 10
        CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY = 11
        CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING = 99
        INVALID_COUNTRY_CODE = 13
        INVALID_APP_ID = 14
        MISSING_ATTRIBUTES_FOR_FIELDS = 15
        INVALID_TYPE_ID = 16
        INVALID_EMAIL_ADDRESS = 17
        INVALID_HTTPS_URL = 18
        MISSING_DELIVERY_ADDRESS = 19
        START_DATE_AFTER_END_DATE = 20
        MISSING_FEED_ITEM_START_TIME = 21
        MISSING_FEED_ITEM_END_TIME = 22
        MISSING_FEED_ITEM_ID = 23
        VANITY_PHONE_NUMBER_NOT_ALLOWED = 24
        INVALID_REVIEW_EXTENSION_SNIPPET = 25
        INVALID_NUMBER_FORMAT = 26
        INVALID_DATE_FORMAT = 27
        INVALID_PRICE_FORMAT = 28
        UNKNOWN_PLACEHOLDER_FIELD = 29
        MISSING_ENHANCED_SITELINK_DESCRIPTION_LINE = 30
        REVIEW_EXTENSION_SOURCE_INELIGIBLE = 31
        HYPHENS_IN_REVIEW_EXTENSION_SNIPPET = 32
        DOUBLE_QUOTES_IN_REVIEW_EXTENSION_SNIPPET = 33
        QUOTES_IN_REVIEW_EXTENSION_SNIPPET = 34
        INVALID_FORM_ENCODED_PARAMS = 35
        INVALID_URL_PARAMETER_NAME = 36
        NO_GEOCODING_RESULT = 37
        SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT = 38
        CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED = 39
        INVALID_PLACEHOLDER_FIELD_ID = 40
        INVALID_URL_TAG = 41
        LIST_TOO_LONG = 42
        INVALID_ATTRIBUTES_COMBINATION = 43
        DUPLICATE_VALUES = 44
        INVALID_CALL_CONVERSION_ACTION_ID = 45
        CANNOT_SET_WITHOUT_FINAL_URLS = 46
        APP_ID_DOESNT_EXIST_IN_APP_STORE = 47
        INVALID_FINAL_URL = 48
        INVALID_TRACKING_URL = 49
        INVALID_FINAL_URL_FOR_APP_DOWNLOAD_URL = 50
        LIST_TOO_SHORT = 51
        INVALID_USER_ACTION = 52
        INVALID_TYPE_NAME = 53
        INVALID_EVENT_CHANGE_STATUS = 54
        INVALID_SNIPPETS_HEADER = 55
        INVALID_ANDROID_APP_LINK = 56
        NUMBER_TYPE_WITH_CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY = 57
        RESERVED_KEYWORD_OTHER = 58
        DUPLICATE_OPTION_LABELS = 59
        DUPLICATE_OPTION_PREFILLS = 60
        UNEQUAL_LIST_LENGTHS = 61
        INCONSISTENT_CURRENCY_CODES = 62
        PRICE_EXTENSION_HAS_DUPLICATED_HEADERS = 63
        ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION = 64
        PRICE_EXTENSION_HAS_TOO_FEW_ITEMS = 65
        UNSUPPORTED_VALUE = 66
        INVALID_FINAL_MOBILE_URL = 67
        INVALID_KEYWORDLESS_AD_RULE_LABEL = 68
        VALUE_TRACK_PARAMETER_NOT_SUPPORTED = 69
        UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE = 70
        INVALID_IOS_APP_LINK = 71
        MISSING_IOS_APP_LINK_OR_IOS_APP_STORE_ID = 72
        PROMOTION_INVALID_TIME = 73
        PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF = 74
        PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT = 75
        TOO_MANY_DECIMAL_PLACES_SPECIFIED = 76
        AD_CUSTOMIZERS_NOT_ALLOWED = 77
        INVALID_LANGUAGE_CODE = 78
        UNSUPPORTED_LANGUAGE = 79
        IF_FUNCTION_NOT_ALLOWED = 80
        INVALID_FINAL_URL_SUFFIX = 81
        INVALID_TAG_IN_FINAL_URL_SUFFIX = 82
        INVALID_FINAL_URL_SUFFIX_FORMAT = 83
        CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED = 84
        ONLY_ONE_DELIVERY_OPTION_IS_ALLOWED = 85
        NO_DELIVERY_OPTION_IS_SET = 86
        INVALID_CONVERSION_REPORTING_STATE = 87
        IMAGE_SIZE_WRONG = 88
        EMAIL_DELIVERY_NOT_AVAILABLE_IN_COUNTRY = 89
        AUTO_REPLY_NOT_AVAILABLE_IN_COUNTRY = 90
        INVALID_LATITUDE_VALUE = 91
        INVALID_LONGITUDE_VALUE = 92
        TOO_MANY_LABELS = 93
        INVALID_IMAGE_URL = 94
        MISSING_LATITUDE_VALUE = 95
        MISSING_LONGITUDE_VALUE = 96
        ADDRESS_NOT_FOUND = 97
        ADDRESS_NOT_TARGETABLE = 98
        INVALID_ASSET_ID = 100
        INCOMPATIBLE_ASSET_TYPE = 101
        IMAGE_ERROR_UNEXPECTED_SIZE = 102
        IMAGE_ERROR_ASPECT_RATIO_NOT_ALLOWED = 103
        IMAGE_ERROR_FILE_TOO_LARGE = 104
        IMAGE_ERROR_FORMAT_NOT_ALLOWED = 105
        IMAGE_ERROR_CONSTRAINTS_VIOLATED = 106
        IMAGE_ERROR_SERVER_ERROR = 107


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ContentLabelTypeEnum",
    },
)


class ContentLabelTypeEnum(proto.Message):
    r"""Container for enum describing content label types in
    ContentLabel.

    """

    class ContentLabelType(proto.Enum):
        r"""Enum listing the content label types supported by
        ContentLabel criterion.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            SEXUALLY_SUGGESTIVE (2):
                Sexually suggestive content.
            BELOW_THE_FOLD (3):
                Below the fold placement.
            PARKED_DOMAIN (4):
                Parked domain.
            JUVENILE (6):
                Juvenile, gross & bizarre content.
            PROFANITY (7):
                Profanity & rough language.
            TRAGEDY (8):
                Death & tragedy.
            VIDEO (9):
                Video.
            VIDEO_RATING_DV_G (10):
                Content rating: G.
            VIDEO_RATING_DV_PG (11):
                Content rating: PG.
            VIDEO_RATING_DV_T (12):
                Content rating: T.
            VIDEO_RATING_DV_MA (13):
                Content rating: MA.
            VIDEO_NOT_YET_RATED (14):
                Content rating: not yet rated.
            EMBEDDED_VIDEO (15):
                Embedded video.
            LIVE_STREAMING_VIDEO (16):
                Live streaming video.
            SOCIAL_ISSUES (17):
                Sensitive social issues.
            BRAND_SUITABILITY_CONTENT_FOR_FAMILIES (18):
                Content that's suitable for families to view
                together, including Made for Kids videos on
                YouTube.
            BRAND_SUITABILITY_GAMES_FIGHTING (19):
                Video games that simulate hand-to-hand
                fighting or combat with the use of modern or
                medieval weapons.
            BRAND_SUITABILITY_GAMES_MATURE (20):
                Video games that feature mature content, such
                as violence, inappropriate language, or sexual
                suggestiveness.
            BRAND_SUITABILITY_HEALTH_SENSITIVE (21):
                Health content that people might find
                sensitive or upsetting, such as medical
                procedures or images and descriptions of various
                medical conditions.
            BRAND_SUITABILITY_HEALTH_SOURCE_UNDETERMINED (22):
                Health content from sources that may provide
                accurate information but aren't as commonly
                cited as other, more well-known sources.
            BRAND_SUITABILITY_NEWS_RECENT (23):
                News content that's been recently announced,
                regardless of the themes or people being
                reported on.
            BRAND_SUITABILITY_NEWS_SENSITIVE (24):
                News content that people might find sensitive
                or upsetting, such as crimes, accidents, and
                natural incidents, or commentary on potentially
                controversial social and political issues.
            BRAND_SUITABILITY_NEWS_SOURCE_NOT_FEATURED (25):
                News content from sources that aren't
                featured on Google News or YouTube News.
            BRAND_SUITABILITY_POLITICS (26):
                Political content, such as political
                statements made by well-known politicians,
                political elections, or events widely perceived
                to be political in nature.
            BRAND_SUITABILITY_RELIGION (27):
                Content with religious themes, such as
                religious teachings or customs, holy sites or
                places of worship, well-known religious figures
                or people dressed in religious attire, or
                religious opinions on social and political
                issues.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        SEXUALLY_SUGGESTIVE = 2
        BELOW_THE_FOLD = 3
        PARKED_DOMAIN = 4
        JUVENILE = 6
        PROFANITY = 7
        TRAGEDY = 8
        VIDEO = 9
        VIDEO_RATING_DV_G = 10
        VIDEO_RATING_DV_PG = 11
        VIDEO_RATING_DV_T = 12
        VIDEO_RATING_DV_MA = 13
        VIDEO_NOT_YET_RATED = 14
        EMBEDDED_VIDEO = 15
        LIVE_STREAMING_VIDEO = 16
        SOCIAL_ISSUES = 17
        BRAND_SUITABILITY_CONTENT_FOR_FAMILIES = 18
        BRAND_SUITABILITY_GAMES_FIGHTING = 19
        BRAND_SUITABILITY_GAMES_MATURE = 20
        BRAND_SUITABILITY_HEALTH_SENSITIVE = 21
        BRAND_SUITABILITY_HEALTH_SOURCE_UNDETERMINED = 22
        BRAND_SUITABILITY_NEWS_RECENT = 23
        BRAND_SUITABILITY_NEWS_SENSITIVE = 24
        BRAND_SUITABILITY_NEWS_SOURCE_NOT_FEATURED = 25
        BRAND_SUITABILITY_POLITICS = 26
        BRAND_SUITABILITY_RELIGION = 27


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example adds a call asset to a specific account."""


import argparse
from typing import Optional
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.types.asset_service import AssetOperation
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.common.types.criteria import AdScheduleInfo
from google.ads.googleads.v22.services.types.customer_asset_service import (
    CustomerAssetOperation,
)
from google.ads.googleads.v22.resources.types.customer_asset import (
    CustomerAsset,
)

# Country code is a two-letter ISO-3166 code, for a list of all codes see:
# https://developers.google.com/google-ads/api/reference/data/codes-formats#expandable-17
_DEFAULT_PHONE_COUNTRY: str = "US"


def main(
    client: GoogleAdsClient,
    customer_id: str,
    phone_number: str,
    phone_country: str,
    conversion_action_id: Optional[str],
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        phone_number: a phone number for your business, e.g. '(800) 555-0100'.
        phone_country: a two-letter ISO-3166 code.
        conversion_action_id: an ID for a conversion action.
    """
    asset_resource_name: str = add_call_asset(
        client, customer_id, phone_number, phone_country, conversion_action_id
    )
    link_asset_to_account(client, customer_id, asset_resource_name)


def add_call_asset(
    client: GoogleAdsClient,
    customer_id: str,
    phone_number: str,
    phone_country: str,
    conversion_action_id: Optional[str],
) -> str:
    """Creates a new asset for the call.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        phone_number: a phone number for your business, e.g. '(800) 555-0100'.
        phone_country: a two-letter ISO-3166 code.
        conversion_action_id: an ID for a conversion action.

    Returns:
        a resource name for a new call asset.
    """
    operation: AssetOperation = client.get_type("AssetOperation")
    # Creates the call asset.
    asset: Asset = operation.create.call_asset
    asset.country_code = phone_country
    asset.phone_number = phone_number
    # Optional: Specifies day and time intervals for which the asset may serve.
    ad_schedule: AdScheduleInfo = client.get_type("AdScheduleInfo")
    # Sets the day of this schedule as Monday.
    ad_schedule.day_of_week = client.enums.DayOfWeekEnum.MONDAY
    # Sets the start hour to 9am.
    ad_schedule.start_hour = 9
    # Sets the end hour to 5pm.
    ad_schedule.end_hour = 17
    # Sets the start and end minute of zero, for example: 9:00 and 5:00.
    ad_schedule.start_minute = client.enums.MinuteOfHourEnum.ZERO
    ad_schedule.end_minute = client.enums.MinuteOfHourEnum.ZERO
    # Appends the ad schedule to the list of ad schedule targets on the asset.
    asset.ad_schedule_targets.append(ad_schedule)

    # Sets the conversion action ID if provided.
    if conversion_action_id:
        googleads_service = client.get_service("GoogleAdsService")
        asset.call_conversion_action = googleads_service.conversion_action_path(
            customer_id, conversion_action_id
        )
        asset.call_conversion_reporting_state = (
            client.enums.CallConversionReportingStateEnum.USE_RESOURCE_LEVEL_CALL_CONVERSION_ACTION
        )

    # Issues a mutate request to add the asset.
    asset_service = client.get_service("AssetService")
    response = asset_service.mutate_assets(
        customer_id=customer_id, operations=[operation]
    )
    resource_name: str = response.results[0].resource_name
    print(f"Created a call asset with resource name: '{resource_name}'")

    return resource_name


def link_asset_to_account(
    client: GoogleAdsClient, customer_id: str, asset_resource_name: str
) -> None:
    """Links the call asset at the account level to serve in eligible campaigns.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        asset_resource_name: a resource name for the call asset.
    """
    operation: CustomerAssetOperation = client.get_type(
        "CustomerAssetOperation"
    )
    customer_asset: CustomerAsset = operation.create
    customer_asset.asset = asset_resource_name
    customer_asset.field_type = client.enums.AssetFieldTypeEnum.CALL

    customer_asset_service = client.get_service("CustomerAssetService")
    response = customer_asset_service.mutate_customer_assets(
        customer_id=customer_id, operations=[operation]
    )
    resource_name: str = response.results[0].resource_name
    print(f"Created a customer asset with resource name: '{resource_name}'")


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=("Adds a call asset to a specific account.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-n",
        "--phone_number",
        type=str,
        required=True,
        help=("A phone number for your business, e.g. '(800) 555-0100'"),
    )
    parser.add_argument(
        "-p",
        "--phone_country",
        type=str,
        default=_DEFAULT_PHONE_COUNTRY,
        help=(
            "A two-letter ISO-3166 code representing a country code, for a "
            "list of all codes see: "
            "https://developers.google.com/google-ads/api/reference/data/codes-formats#expandable-17"
        ),
    )
    parser.add_argument(
        "-v",
        "--conversion_action_id",
        type=str,
        help=("An optional conversion action ID to attribute conversions to."),
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.phone_number,
            args.phone_country,
            args.conversion_action_id,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdGroupAdPrimaryStatusEnum",
    },
)


class AdGroupAdPrimaryStatusEnum(proto.Message):
    r"""Ad Group Ad Primary Status. Provides insight into why an ad
    group ad is not serving or not serving optimally.

    """

    class AdGroupAdPrimaryStatus(proto.Enum):
        r"""The possible statuses of an AdGroupAd.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ELIGIBLE (2):
                The ad group ad is eligible to serve.
            PAUSED (3):
                The ad group ad is paused.
            REMOVED (4):
                The ad group ad is removed.
            PENDING (5):
                The ad cannot serve now, but may serve later
                without advertiser action.
            LIMITED (6):
                The ad group ad is serving in a limited
                capacity.
            NOT_ELIGIBLE (7):
                The ad group ad is not eligible to serve.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ELIGIBLE = 2
        PAUSED = 3
        REMOVED = 4
        PENDING = 5
        LIMITED = 6
        NOT_ELIGIBLE = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
from typing import Callable, MutableSequence, Optional, Sequence, Tuple, Union

from google.ads.googleads.v20 import gapic_version as package_version

from google.api_core.client_options import ClientOptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf


try:
    OptionalRetry = Union[
        retries.AsyncRetry, gapic_v1.method._MethodDefault, None
    ]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.ads.googleads.v20.services.services.experiment_service import pagers
from google.ads.googleads.v20.services.types import experiment_service
from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
from .transports.base import ExperimentServiceTransport, DEFAULT_CLIENT_INFO
from .client import ExperimentServiceClient

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class ExperimentServiceAsyncClient:
    """Service to manage experiments."""

    _client: ExperimentServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = ExperimentServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = ExperimentServiceClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = (
        ExperimentServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    )
    _DEFAULT_UNIVERSE = ExperimentServiceClient._DEFAULT_UNIVERSE

    campaign_path = staticmethod(ExperimentServiceClient.campaign_path)
    parse_campaign_path = staticmethod(
        ExperimentServiceClient.parse_campaign_path
    )
    campaign_budget_path = staticmethod(
        ExperimentServiceClient.campaign_budget_path
    )
    parse_campaign_budget_path = staticmethod(
        ExperimentServiceClient.parse_campaign_budget_path
    )
    experiment_path = staticmethod(ExperimentServiceClient.experiment_path)
    parse_experiment_path = staticmethod(
        ExperimentServiceClient.parse_experiment_path
    )
    common_billing_account_path = staticmethod(
        ExperimentServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        ExperimentServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        ExperimentServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        ExperimentServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        ExperimentServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        ExperimentServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        ExperimentServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        ExperimentServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        ExperimentServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        ExperimentServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            ExperimentServiceAsyncClient: The constructed client.
        """
        return ExperimentServiceClient.from_service_account_info.__func__(ExperimentServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            ExperimentServiceAsyncClient: The constructed client.
        """
        return ExperimentServiceClient.from_service_account_file.__func__(ExperimentServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return ExperimentServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> ExperimentServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            ExperimentServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = ExperimentServiceClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[
                str,
                ExperimentServiceTransport,
                Callable[..., ExperimentServiceTransport],
            ]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the experiment service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,ExperimentServiceTransport,Callable[..., ExperimentServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the ExperimentServiceTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = ExperimentServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.ads.googleads.v20.services.ExperimentServiceAsyncClient`.",
                extra=(
                    {
                        "serviceName": "google.ads.googleads.v20.services.ExperimentService",
                        "universeDomain": getattr(
                            self._client._transport._credentials,
                            "universe_domain",
                            "",
                        ),
                        "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                        "credentialsInfo": getattr(
                            self.transport._credentials,
                            "get_cred_info",
                            lambda: None,
                        )(),
                    }
                    if hasattr(self._client._transport, "_credentials")
                    else {
                        "serviceName": "google.ads.googleads.v20.services.ExperimentService",
                        "credentialsType": None,
                    }
                ),
            )

    async def mutate_experiments(
        self,
        request: Optional[
            Union[experiment_service.MutateExperimentsRequest, dict]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        operations: Optional[
            MutableSequence[experiment_service.ExperimentOperation]
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> experiment_service.MutateExperimentsResponse:
        r"""Creates, updates, or removes experiments. Operation statuses are
        returned.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `ExperimentError <>`__
        `HeaderError <>`__ `InternalError <>`__ `QuotaError <>`__
        `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v20.services.types.MutateExperimentsRequest, dict]]):
                The request object. Request message for
                [ExperimentService.MutateExperiments][google.ads.googleads.v20.services.ExperimentService.MutateExperiments].
            customer_id (:class:`str`):
                Required. The ID of the customer
                whose experiments are being modified.

                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            operations (:class:`MutableSequence[google.ads.googleads.v20.services.types.ExperimentOperation]`):
                Required. The list of operations to
                perform on individual experiments.

                This corresponds to the ``operations`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v20.services.types.MutateExperimentsResponse:
                Response message for experiment
                mutate.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [customer_id, operations]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, experiment_service.MutateExperimentsRequest):
            request = experiment_service.MutateExperimentsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if customer_id is not None:
            request.customer_id = customer_id
        if operations:
            request.operations.extend(operations)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.mutate_experiments
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def end_experiment(
        self,
        request: Optional[
            Union[experiment_service.EndExperimentRequest, dict]
        ] = None,
        *,
        experiment: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> None:
        r"""Immediately ends an experiment, changing the experiment's
        scheduled end date and without waiting for end of day. End date
        is updated to be the time of the request.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `ExperimentError <>`__
        `HeaderError <>`__ `InternalError <>`__ `QuotaError <>`__
        `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v20.services.types.EndExperimentRequest, dict]]):
                The request object. Request message for
                [ExperimentService.EndExperiment][google.ads.googleads.v20.services.ExperimentService.EndExperiment].
            experiment (:class:`str`):
                Required. The resource name of the
                campaign experiment to end.

                This corresponds to the ``experiment`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [experiment]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, experiment_service.EndExperimentRequest):
            request = experiment_service.EndExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if experiment is not None:
            request.experiment = experiment

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.end_experiment
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("experiment", request.experiment),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def list_experiment_async_errors(
        self,
        request: Optional[
            Union[experiment_service.ListExperimentAsyncErrorsRequest, dict]
        ] = None,
        *,
        resource_name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> pagers.ListExperimentAsyncErrorsAsyncPager:
        r"""Returns all errors that occurred during the last Experiment
        update (either scheduling or promotion). Supports standard list
        paging.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `HeaderError <>`__
        `InternalError <>`__ `QuotaError <>`__ `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v20.services.types.ListExperimentAsyncErrorsRequest, dict]]):
                The request object. Request message for
                [ExperimentService.ListExperimentAsyncErrors][google.ads.googleads.v20.services.ExperimentService.ListExperimentAsyncErrors].
            resource_name (:class:`str`):
                Required. The name of the experiment
                from which to retrieve the async errors.

                This corresponds to the ``resource_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v20.services.services.experiment_service.pagers.ListExperimentAsyncErrorsAsyncPager:
                Response message for
                   [ExperimentService.ListExperimentAsyncErrors][google.ads.googleads.v20.services.ExperimentService.ListExperimentAsyncErrors].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [resource_name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, experiment_service.ListExperimentAsyncErrorsRequest
        ):
            request = experiment_service.ListExperimentAsyncErrorsRequest(
                request
            )

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if resource_name is not None:
            request.resource_name = resource_name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_experiment_async_errors
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("resource_name", request.resource_name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListExperimentAsyncErrorsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def graduate_experiment(
        self,
        request: Optional[
            Union[experiment_service.GraduateExperimentRequest, dict]
        ] = None,
        *,
        experiment: Optional[str] = None,
        campaign_budget_mappings: Optional[
            MutableSequence[experiment_service.CampaignBudgetMapping]
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> None:
        r"""Graduates an experiment to a full campaign.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `ExperimentError <>`__
        `HeaderError <>`__ `InternalError <>`__ `MutateError <>`__
        `QuotaError <>`__ `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v20.services.types.GraduateExperimentRequest, dict]]):
                The request object. Request message for
                [ExperimentService.GraduateExperiment][google.ads.googleads.v20.services.ExperimentService.GraduateExperiment].
            experiment (:class:`str`):
                Required. The experiment to be
                graduated.

                This corresponds to the ``experiment`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            campaign_budget_mappings (:class:`MutableSequence[google.ads.googleads.v20.services.types.CampaignBudgetMapping]`):
                Required. List of campaign budget
                mappings for graduation. Each campaign
                that appears here will graduate, and
                will be assigned a new budget that is
                paired with it in the mapping. The
                maximum size is one.

                This corresponds to the ``campaign_budget_mappings`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [experiment, campaign_budget_mappings]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, experiment_service.GraduateExperimentRequest
        ):
            request = experiment_service.GraduateExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if experiment is not None:
            request.experiment = experiment
        if campaign_budget_mappings:
            request.campaign_budget_mappings.extend(campaign_budget_mappings)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.graduate_experiment
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("experiment", request.experiment),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def schedule_experiment(
        self,
        request: Optional[
            Union[experiment_service.ScheduleExperimentRequest, dict]
        ] = None,
        *,
        resource_name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Schedule an experiment. The in design campaign will be converted
        into a real campaign (called the experiment campaign) that will
        begin serving ads if successfully created.

        The experiment is scheduled immediately with status
        INITIALIZING. This method returns a long running operation that
        tracks the forking of the in design campaign. If the forking
        fails, a list of errors can be retrieved using the
        ListExperimentAsyncErrors method. The operation's metadata will
        be a string containing the resource name of the created
        experiment.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `ExperimentError <>`__
        `DatabaseError <>`__ `DateError <>`__ `DateRangeError <>`__
        `FieldError <>`__ `HeaderError <>`__ `InternalError <>`__
        `QuotaError <>`__ `RangeError <>`__ `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v20.services.types.ScheduleExperimentRequest, dict]]):
                The request object. Request message for
                [ExperimentService.ScheduleExperiment][google.ads.googleads.v20.services.ExperimentService.ScheduleExperiment].
            resource_name (:class:`str`):
                Required. The scheduled experiment.
                This corresponds to the ``resource_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [resource_name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, experiment_service.ScheduleExperimentRequest
        ):
            request = experiment_service.ScheduleExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if resource_name is not None:
            request.resource_name = resource_name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.schedule_experiment
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("resource_name", request.resource_name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=experiment_service.ScheduleExperimentMetadata,
        )

        # Done; return the response.
        return response

    async def promote_experiment(
        self,
        request: Optional[
            Union[experiment_service.PromoteExperimentRequest, dict]
        ] = None,
        *,
        resource_name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Promotes the trial campaign thus applying changes in the trial
        campaign to the base campaign. This method returns a long
        running operation that tracks the promotion of the experiment
        campaign. If it fails, a list of errors can be retrieved using
        the ListExperimentAsyncErrors method. The operation's metadata
        will be a string containing the resource name of the created
        experiment.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `ExperimentError <>`__
        `HeaderError <>`__ `InternalError <>`__ `QuotaError <>`__
        `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v20.services.types.PromoteExperimentRequest, dict]]):
                The request object. Request message for
                [ExperimentService.PromoteExperiment][google.ads.googleads.v20.services.ExperimentService.PromoteExperiment].
            resource_name (:class:`str`):
                Required. The resource name of the
                experiment to promote.

                This corresponds to the ``resource_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [resource_name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, experiment_service.PromoteExperimentRequest):
            request = experiment_service.PromoteExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if resource_name is not None:
            request.resource_name = resource_name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.promote_experiment
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("resource_name", request.resource_name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=experiment_service.PromoteExperimentMetadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "ExperimentServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("ExperimentServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "AuthorizationErrorEnum",
    },
)


class AuthorizationErrorEnum(proto.Message):
    r"""Container for enum describing possible authorization errors."""

    class AuthorizationError(proto.Enum):
        r"""Enum describing possible authorization errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            USER_PERMISSION_DENIED (2):
                User doesn't have permission to access customer. Note: If
                you're accessing a client customer, the manager's customer
                ID must be set in the ``login-customer-id`` header. Learn
                more at
                https://developers.google.com/google-ads/api/docs/concepts/call-structure#cid
            DEVELOPER_TOKEN_NOT_ON_ALLOWLIST (13):
                The developer token is not on the allow-list.
            DEVELOPER_TOKEN_PROHIBITED (4):
                The developer token is not allowed with the
                project sent in the request.
            PROJECT_DISABLED (5):
                The Google Cloud project sent in the request
                does not have permission to access the api.
            AUTHORIZATION_ERROR (6):
                Authorization of the client failed.
            ACTION_NOT_PERMITTED (7):
                The user does not have permission to perform
                this action (for example, ADD, UPDATE, REMOVE)
                on the resource or call a method.
            INCOMPLETE_SIGNUP (8):
                Signup not complete.
            CUSTOMER_NOT_ENABLED (24):
                The customer account can't be accessed
                because it is not yet enabled or has been
                deactivated.
            MISSING_TOS (9):
                The developer must sign the terms of service.
                They can be found here:
                ads.google.com/aw/apicenter
            DEVELOPER_TOKEN_NOT_APPROVED (10):
                The developer token is only approved for use
                with test accounts. To access non-test accounts,
                apply for Basic or Standard access.
            INVALID_LOGIN_CUSTOMER_ID_SERVING_CUSTOMER_ID_COMBINATION (11):
                The login customer specified does not have
                access to the account specified, so the request
                is invalid.
            SERVICE_ACCESS_DENIED (12):
                The developer specified does not have access
                to the service.
            ACCESS_DENIED_FOR_ACCOUNT_TYPE (25):
                The customer (or login customer) isn't in
                Google Ads. It belongs to another ads system.
            METRIC_ACCESS_DENIED (26):
                The developer does not have access to the
                metrics queried.
            CLOUD_PROJECT_NOT_UNDER_ORGANIZATION (27):
                The Google Cloud project is not under the
                required organization.
            ACTION_NOT_PERMITTED_FOR_SUSPENDED_ACCOUNT (28):
                The user does not have permission to perform
                this action on the resource or method because
                the Google Ads account is suspended.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        USER_PERMISSION_DENIED = 2
        DEVELOPER_TOKEN_NOT_ON_ALLOWLIST = 13
        DEVELOPER_TOKEN_PROHIBITED = 4
        PROJECT_DISABLED = 5
        AUTHORIZATION_ERROR = 6
        ACTION_NOT_PERMITTED = 7
        INCOMPLETE_SIGNUP = 8
        CUSTOMER_NOT_ENABLED = 24
        MISSING_TOS = 9
        DEVELOPER_TOKEN_NOT_APPROVED = 10
        INVALID_LOGIN_CUSTOMER_ID_SERVING_CUSTOMER_ID_COMBINATION = 11
        SERVICE_ACCESS_DENIED = 12
        ACCESS_DENIED_FOR_ACCOUNT_TYPE = 25
        METRIC_ACCESS_DENIED = 26
        CLOUD_PROJECT_NOT_UNDER_ORGANIZATION = 27
        ACTION_NOT_PERMITTED_FOR_SUSPENDED_ACCOUNT = 28


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "ShoppingPerformanceView",
    },
)


class ShoppingPerformanceView(proto.Message):
    r"""Shopping performance view.
    Provides Shopping campaign statistics aggregated at several
    product dimension levels. Product dimension values from Merchant
    Center such as brand, category, custom attributes, product
    condition and product type will reflect the state of each
    dimension as of the date and time when the corresponding event
    was recorded.

    Attributes:
        resource_name (str):
            Output only. The resource name of the Shopping performance
            view. Shopping performance view resource names have the
            form: ``customers/{customer_id}/shoppingPerformanceView``
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Handles RateExceededError in an application.

This code example runs 5 requests sequentially, each request attempting to
validate 100 keywords. While it is unlikely that running these requests would
trigger a rate exceeded error, substantially increasing the number of requests
may have that effect. Note that this example is for illustrative purposes only,
and you shouldn't intentionally try to trigger a rate exceed error in your
application.
"""


import argparse
from time import sleep
from typing import List, Any

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.errors.types.quota_error import QuotaErrorEnum
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
    MutateAdGroupCriteriaRequest,
    MutateAdGroupCriteriaResponse,
)

# Number of requests to be run.
NUM_REQUESTS: int = 5
# Number of keywords to be validated in each API call.
NUM_KEYWORDS: int = 100
# Number of retries to be run in case of a RateExceededError.
NUM_RETRIES: int = 3
# Minimum number of seconds to wait before a retry.
RETRY_SECONDS: int = 10


def main(client: GoogleAdsClient, customer_id: str, ad_group_id: str) -> None:
    """Runs the example code, which shows how to handle rate exceeded errors.

    Args:
        client:  An initialized GoogleAdsClient instance.
        customer_id: A valid customer account ID.
        ad_group_id: The ad group ID to validate keywords from.
    """
    quota_error_enum: QuotaErrorEnum = client.get_type(
        "QuotaErrorEnum"
    ).QuotaError
    resource_exhausted: QuotaErrorEnum = quota_error_enum.RESOURCE_EXHAUSTED
    temp_resource_exhausted: QuotaErrorEnum = (
        quota_error_enum.RESOURCE_TEMPORARILY_EXHAUSTED
    )

    for i in range(NUM_REQUESTS):
        operations: List[AdGroupCriterionOperation] = (
            create_ad_group_criterion_operations(
                client, customer_id, ad_group_id, i
            )
        )

        try:
            retry_count: int = 0
            retry_seconds: int = RETRY_SECONDS

            while retry_count < NUM_RETRIES:
                try:
                    request_mutate_and_display_result(
                        client, customer_id, operations
                    )
                    break
                except GoogleAdsException as ex:
                    has_rate_exceeded_error: bool = False
                    for googleads_error in ex.failure.errors:
                        # Checks if any of the errors are
                        # QuotaError.RESOURCE_EXHAUSTED or
                        # QuotaError.RESOURCE_TEMPORARILY_EXHAUSTED.
                        quota_error: QuotaErrorEnum = (
                            googleads_error.error_code.quota_error
                        )
                        if (
                            quota_error == resource_exhausted
                            or quota_error == temp_resource_exhausted
                        ):
                            print(
                                "Received rate exceeded error, retry after"
                                f"{retry_seconds} seconds."
                            )
                            sleep(retry_seconds)
                            has_rate_exceeded_error = True
                            retry_count += 1
                            # Here exponential backoff is employed to ensure
                            # the account doesn't get rate limited by making
                            # too many requests too quickly. This increases the
                            # time to wait between requests by a factor of 2.
                            retry_seconds *= 2
                            break
                    # Bubbles up when there is not a RateExceededError
                    if not has_rate_exceeded_error:
                        raise ex
                finally:
                    if retry_count == NUM_RETRIES:
                        raise Exception(
                            "Could not recover after making "
                            f"{retry_count} attempts."
                        )
        except Exception as ex:
            # Prints any unhandled exception and bubbles up.
            print(f"Failed to validate keywords: {ex}")
            raise ex


def create_ad_group_criterion_operations(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    request_index: int,
) -> List[AdGroupCriterionOperation]:
    """Creates ad group criterion operations.

    The number of operations created depends on the number of keywords this
    example should remove. That value is configurable via the NUM_KEYWORDS
    variable.

    Args:
        client:  An initialized GoogleAdsClient instance.
        customer_id: A valid customer account ID.
        ad_group_id: An ID for an AdGroup.
        request_index: The number from a sequence of requests in which these
            operations will be sent.

    Returns:
        A list of AdGroupCriterionOperation instances.
    """
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )
    status: Any = client.enums.AdGroupCriterionStatusEnum.ENABLED
    match_type: Any = client.enums.KeywordMatchTypeEnum.EXACT

    operations: List[AdGroupCriterionOperation] = []
    for i in range(NUM_KEYWORDS):
        ad_group_criterion_operation: AdGroupCriterionOperation = (
            client.get_type("AdGroupCriterionOperation")
        )
        ad_group_criterion: AdGroupCriterion = (
            ad_group_criterion_operation.create
        )
        ad_group_criterion.ad_group = ad_group_service.ad_group_path(
            customer_id, ad_group_id
        )
        ad_group_criterion.status = status
        ad_group_criterion.keyword.text = (
            f"mars cruise req {request_index} seed {i}"
        )
        ad_group_criterion.keyword.match_type = match_type
        operations.append(ad_group_criterion_operation)

    return operations


def request_mutate_and_display_result(
    client: GoogleAdsClient,
    customer_id: str,
    operations: List[AdGroupCriterionOperation],
) -> None:
    """Mutates a set of ad group criteria as a dry-run and displays the results.

    The request is sent with validate_only set to true, so no actual mutations
    will be made in the API.

    Args:
        client:  An initialized GoogleAdsClient instance.
        customer_id: A valid customer account ID.
        operations: a list of AdGroupCriterionOperation instances.
    """
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )
    request: MutateAdGroupCriteriaRequest = client.get_type(
        "MutateAdGroupCriteriaRequest"
    )
    request.customer_id = customer_id
    request.operations = operations
    request.validate_only = True
    response: MutateAdGroupCriteriaResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(request=request)
    )
    print(f"Added {len(response.results)} ad group criteria:")
    for ad_group_criterion in response.results:
        print(f"Resource name: '{ad_group_criterion.resource_name}'")


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Handles RateExceededError in an application.."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=True,
        help="The ID of an ad group belonging to the given customer.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    main(googleads_client, args.customer_id, args.ad_group_id)



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Demonstrates how to find and remove shared sets, and shared set criteria."""


import argparse
import sys
from typing import List

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.criteria import KeywordInfo
from google.ads.googleads.v22.enums.types.criterion_type import (
    CriterionTypeEnum,
)
from google.ads.googleads.v22.errors.types.errors import GoogleAdsError
from google.ads.googleads.v22.resources.types.shared_criterion import (
    SharedCriterion,
)
from google.ads.googleads.v22.resources.types.shared_set import SharedSet
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.services.shared_criterion_service import (
    SharedCriterionServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    GoogleAdsRow,
    SearchGoogleAdsRequest,
    SearchGoogleAdsResponse,
)
from google.ads.googleads.v22.services.types.shared_criterion_service import (
    MutateSharedCriteriaResponse,
    MutateSharedCriterionResult,
    SharedCriterionOperation,
)


def main(client: GoogleAdsClient, customer_id: str, campaign_id: str) -> None:
    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")
    shared_criterion_service: SharedCriterionServiceClient = client.get_service(
        "SharedCriterionService"
    )

    # First, retrieve all shared sets associated with the campaign.
    shared_sets_query: str = f"""
        SELECT
          shared_set.id,
          shared_set.name
        FROM campaign_shared_set
        WHERE campaign.id = {campaign_id}"""

    try:
        shared_set_search_request: SearchGoogleAdsRequest = client.get_type(
            "SearchGoogleAdsRequest"
        )
        shared_set_search_request.customer_id = customer_id
        shared_set_search_request.query = shared_sets_query

        shared_set_response: SearchGoogleAdsResponse = ga_service.search(
            request=shared_set_search_request
        )

        shared_set_ids: List[str] = []
        row: GoogleAdsRow
        for row in shared_set_response:
            shared_set: SharedSet = row.shared_set
            shared_set_ids.append(str(shared_set.id))
            print(
                f'Campaign shared set ID "{shared_set.id}" and name '
                f'"{shared_set.name}" was found.'
            )
    except GoogleAdsException as ex:
        handle_googleads_exception(ex)

    # Next, retrieve shared criteria for all found shared sets.
    ids: str = ", ".join(shared_set_ids)
    shared_criteria_query: str = f"""
        SELECT
          shared_criterion.type,
          shared_criterion.keyword.text,
          shared_criterion.keyword.match_type,
          shared_set.id
        FROM shared_criterion
        WHERE shared_set.id IN ({ids})"""

    try:
        shared_criteria_search_request: SearchGoogleAdsRequest = (
            client.get_type("SearchGoogleAdsRequest")
        )
        shared_criteria_search_request.customer_id = customer_id
        shared_criteria_search_request.query = shared_criteria_query

        shared_criteria_response: SearchGoogleAdsResponse = ga_service.search(
            request=shared_criteria_search_request
        )
    except GoogleAdsException as ex:
        handle_googleads_exception(ex)

    criterion_type_enum: CriterionTypeEnum = client.enums.CriterionTypeEnum
    criterion_ids: List[str] = []
    row: GoogleAdsRow
    for row in shared_criteria_response:
        shared_criterion: SharedCriterion = row.shared_criterion
        shared_criterion_resource_name: str = shared_criterion.resource_name

        if shared_criterion.type_ == criterion_type_enum.KEYWORD:
            keyword: KeywordInfo = shared_criterion.keyword
            print(
                "Shared criterion with resource name "
                f'"{shared_criterion_resource_name}" for negative keyword '
                f'with text "{keyword.text}" and match type '
                f'"{keyword.match_type.name}" was found.'
            )

        criterion_ids.append(shared_criterion_resource_name)

    operations: List[SharedCriterionOperation] = []

    # Finally, remove the criteria.
    criteria_id: str
    for criteria_id in criterion_ids:
        shared_criterion_operation: SharedCriterionOperation = client.get_type(
            "SharedCriterionOperation"
        )
        shared_criterion_operation.remove = criteria_id
        operations.append(shared_criterion_operation)

    try:
        response: MutateSharedCriteriaResponse = (
            shared_criterion_service.mutate_shared_criteria(
                customer_id=customer_id, operations=operations
            )
        )

        result: MutateSharedCriterionResult
        for result in response.results:
            print(f'Removed shared criterion "{result.resource_name}".')
    except GoogleAdsException as ex:
        handle_googleads_exception(ex)


def handle_googleads_exception(exception: GoogleAdsException) -> None:
    print(
        f'Request with ID "{exception.request_id}" failed with status '
        f'"{exception.error.code().name}" and includes the following errors:'
    )
    error: GoogleAdsError
    for error in exception.failure.errors:
        print(f'\tError with message "{error.message}".')
        if error.location:
            for field_path_element in error.location.field_path_elements:
                print(f"\t\tOn field: {field_path_element.field_name}")
    sys.exit(1)


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=(
            "Finds shared sets, then finds and removes shared set "
            "criteria under them."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-i", "--campaign_id", type=str, required=True, help="The campaign ID."
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    main(googleads_client, args.customer_id, args.campaign_id)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import fixed_cpm_goal
from google.ads.googleads.v22.enums.types import (
    fixed_cpm_target_frequency_time_unit,
)
from google.ads.googleads.v22.enums.types import target_frequency_time_unit
from google.ads.googleads.v22.enums.types import (
    target_impression_share_location,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.common",
    marshal="google.ads.googleads.v22",
    manifest={
        "Commission",
        "EnhancedCpc",
        "ManualCpa",
        "ManualCpc",
        "ManualCpm",
        "ManualCpv",
        "MaximizeConversions",
        "MaximizeConversionValue",
        "TargetCpa",
        "TargetCpm",
        "TargetCpmTargetFrequencyGoal",
        "TargetImpressionShare",
        "TargetRoas",
        "TargetSpend",
        "PercentCpc",
        "FixedCpm",
        "FixedCpmTargetFrequencyGoalInfo",
        "TargetCpv",
        "TargetCpc",
    },
)


class Commission(proto.Message):
    r"""Commission is an automatic bidding strategy in which the
    advertiser pays a certain portion of the conversion value.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        commission_rate_micros (int):
            Commission rate defines the portion of the conversion value
            that the advertiser will be billed. A commission rate of x
            should be passed into this field as (x \* 1,000,000). For
            example, 106,000 represents a commission rate of 0.106
            (10.6%).

            This field is a member of `oneof`_ ``_commission_rate_micros``.
    """

    commission_rate_micros: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )


class EnhancedCpc(proto.Message):
    r"""An automated bidding strategy that raises bids for clicks that seem
    more likely to lead to a conversion and lowers them for clicks where
    they seem less likely.

    This bidding strategy is deprecated and cannot be created anymore.
    Use ManualCpc with enhanced_cpc_enabled set to true for equivalent
    functionality.

    """


class ManualCpa(proto.Message):
    r"""Manual bidding strategy that allows advertiser to set the bid
    per advertiser-specified action.

    """


class ManualCpc(proto.Message):
    r"""Manual click-based bidding where user pays per click.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        enhanced_cpc_enabled (bool):
            Whether bids are to be enhanced based on
            conversion optimizer data.

            This field is a member of `oneof`_ ``_enhanced_cpc_enabled``.
    """

    enhanced_cpc_enabled: bool = proto.Field(
        proto.BOOL,
        number=2,
        optional=True,
    )


class ManualCpm(proto.Message):
    r"""Manual impression-based bidding where user pays per thousand
    impressions.

    """


class ManualCpv(proto.Message):
    r"""View based bidding where user pays per video view."""


class MaximizeConversions(proto.Message):
    r"""An automated bidding strategy to help get the most
    conversions for your campaigns while spending your budget.

    Attributes:
        cpc_bid_ceiling_micros (int):
            Maximum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. Mutable for portfolio
            bidding strategies only.
        cpc_bid_floor_micros (int):
            Minimum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. Mutable for portfolio
            bidding strategies only.
        target_cpa_micros (int):
            The target cost-per-action (CPA) option. This
            is the average amount that you would like to
            spend per conversion action specified in micro
            units of the bidding strategy's currency. If
            set, the bid strategy will get as many
            conversions as possible at or below the target
            cost-per-action. If the target CPA is not set,
            the bid strategy will aim to achieve the lowest
            possible CPA given the budget.
    """

    cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=2,
    )
    cpc_bid_floor_micros: int = proto.Field(
        proto.INT64,
        number=3,
    )
    target_cpa_micros: int = proto.Field(
        proto.INT64,
        number=4,
    )


class MaximizeConversionValue(proto.Message):
    r"""An automated bidding strategy to help get the most conversion
    value for your campaigns while spending your budget.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        target_roas (float):
            The target return on ad spend (ROAS) option.
            If set, the bid strategy will maximize revenue
            while averaging the target return on ad spend.
            If the target ROAS is high, the bid strategy may
            not be able to spend the full budget. If the
            target ROAS is not set, the bid strategy will
            aim to achieve the highest possible ROAS for the
            budget.
        cpc_bid_ceiling_micros (int):
            Maximum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. Mutable for portfolio
            bidding strategies only.
        cpc_bid_floor_micros (int):
            Minimum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. Mutable for portfolio
            bidding strategies only.
        target_roas_tolerance_percent_millis (int):
            The percent of ROAS(return on advertising
            spend) degradation tolerance allowed to increase
            traffic diversity and conversion volume,
            specified in millis (for example, 10,000 = 10%).
            A value of 10,000 means that the advertiser can
            expect ROAS degradation of up to 10% of the
            specified target ROAS.

            This field is a member of `oneof`_ ``_target_roas_tolerance_percent_millis``.
    """

    target_roas: float = proto.Field(
        proto.DOUBLE,
        number=2,
    )
    cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=3,
    )
    cpc_bid_floor_micros: int = proto.Field(
        proto.INT64,
        number=4,
    )
    target_roas_tolerance_percent_millis: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )


class TargetCpa(proto.Message):
    r"""An automated bid strategy that sets bids to help get as many
    conversions as possible at the target cost-per-acquisition (CPA)
    you set.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        target_cpa_micros (int):
            Average CPA target.
            This target should be greater than or equal to
            minimum billable unit based on the currency for
            the account.

            This field is a member of `oneof`_ ``_target_cpa_micros``.
        cpc_bid_ceiling_micros (int):
            Maximum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. This should only be set
            for portfolio bid strategies.

            This field is a member of `oneof`_ ``_cpc_bid_ceiling_micros``.
        cpc_bid_floor_micros (int):
            Minimum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. This should only be set
            for portfolio bid strategies.

            This field is a member of `oneof`_ ``_cpc_bid_floor_micros``.
    """

    target_cpa_micros: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    cpc_bid_floor_micros: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )


class TargetCpm(proto.Message):
    r"""Target CPM (cost per thousand impressions) is an automated
    bidding strategy that sets bids to optimize performance given
    the target CPM you set.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        target_frequency_goal (google.ads.googleads.v22.common.types.TargetCpmTargetFrequencyGoal):
            Target Frequency bidding goal details.

            This field is a member of `oneof`_ ``goal``.
    """

    target_frequency_goal: "TargetCpmTargetFrequencyGoal" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="goal",
        message="TargetCpmTargetFrequencyGoal",
    )


class TargetCpmTargetFrequencyGoal(proto.Message):
    r"""Target Frequency bidding goal details.

    Attributes:
        target_count (int):
            Target Frequency count representing how many
            times you want to reach a single user.
        time_unit (google.ads.googleads.v22.enums.types.TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit):
            Time window expressing the period over which you want to
            reach the specified target_count.
    """

    target_count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    time_unit: (
        target_frequency_time_unit.TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=target_frequency_time_unit.TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit,
    )


class TargetImpressionShare(proto.Message):
    r"""An automated bidding strategy that sets bids so that a
    certain percentage of search ads are shown at the top of the
    first page (or other targeted location).


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        location (google.ads.googleads.v22.enums.types.TargetImpressionShareLocationEnum.TargetImpressionShareLocation):
            The targeted location on the search results
            page.
        location_fraction_micros (int):
            The chosen fraction of ads to be shown in the
            targeted location in micros. For example, 1%
            equals 10,000.

            This field is a member of `oneof`_ ``_location_fraction_micros``.
        cpc_bid_ceiling_micros (int):
            The highest CPC bid the automated bidding
            system is permitted to specify. This is a
            required field entered by the advertiser that
            sets the ceiling and specified in local micros.

            This field is a member of `oneof`_ ``_cpc_bid_ceiling_micros``.
    """

    location: (
        target_impression_share_location.TargetImpressionShareLocationEnum.TargetImpressionShareLocation
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=target_impression_share_location.TargetImpressionShareLocationEnum.TargetImpressionShareLocation,
    )
    location_fraction_micros: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )


class TargetRoas(proto.Message):
    r"""An automated bidding strategy that helps you maximize revenue
    while averaging a specific target return on ad spend (ROAS).


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        target_roas (float):
            Required. The chosen revenue (based on
            conversion data) per unit of spend. Value must
            be between 0.01 and 1000.0, inclusive.

            This field is a member of `oneof`_ ``_target_roas``.
        cpc_bid_ceiling_micros (int):
            Maximum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. This should only be set
            for portfolio bid strategies.

            This field is a member of `oneof`_ ``_cpc_bid_ceiling_micros``.
        cpc_bid_floor_micros (int):
            Minimum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy. This should only be set
            for portfolio bid strategies.

            This field is a member of `oneof`_ ``_cpc_bid_floor_micros``.
        target_roas_tolerance_percent_millis (int):
            The percent of ROAS(return on advertising
            spend) degradation tolerance allowed to increase
            traffic diversity and conversion volume,
            specified in millis (for example, 10,000 = 10%).
            A value of 10,000 means that the advertiser can
            expect ROAS degradation of up to 10% of the
            specified target ROAS. This field is only
            mutable for portfolio bidding strategies.

            This field is a member of `oneof`_ ``_target_roas_tolerance_percent_millis``.
    """

    target_roas: float = proto.Field(
        proto.DOUBLE,
        number=4,
        optional=True,
    )
    cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    cpc_bid_floor_micros: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    target_roas_tolerance_percent_millis: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )


class TargetSpend(proto.Message):
    r"""An automated bid strategy that sets your bids to help get as
    many clicks as possible within your budget.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        target_spend_micros (int):
            Deprecated: The spend target under which to
            maximize clicks. A TargetSpend bidder will
            attempt to spend the smaller of this value or
            the natural throttling spend amount.
            If not specified, the budget is used as the
            spend target. This field is deprecated and
            should no longer be used. See
            https://ads-developers.googleblog.com/2020/05/reminder-about-sunset-creation-of.html
            for details.

            This field is a member of `oneof`_ ``_target_spend_micros``.
        cpc_bid_ceiling_micros (int):
            Maximum bid limit that can be set by the bid
            strategy. The limit applies to all keywords
            managed by the strategy.

            This field is a member of `oneof`_ ``_cpc_bid_ceiling_micros``.
    """

    target_spend_micros: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )


class PercentCpc(proto.Message):
    r"""A bidding strategy where bids are a fraction of the
    advertised price for some good or service.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        cpc_bid_ceiling_micros (int):
            Maximum bid limit that can be set by the bid strategy. This
            is an optional field entered by the advertiser and specified
            in local micros. Note: A zero value is interpreted in the
            same way as having bid_ceiling undefined.

            This field is a member of `oneof`_ ``_cpc_bid_ceiling_micros``.
        enhanced_cpc_enabled (bool):
            Adjusts the bid for each auction upward or downward,
            depending on the likelihood of a conversion. Individual bids
            may exceed cpc_bid_ceiling_micros, but the average bid
            amount for a campaign should not.

            This field is a member of `oneof`_ ``_enhanced_cpc_enabled``.
    """

    cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    enhanced_cpc_enabled: bool = proto.Field(
        proto.BOOL,
        number=4,
        optional=True,
    )


class FixedCpm(proto.Message):
    r"""Fixed CPM (cost per thousand impression) bidding strategy. A
    manual bidding strategy with a fixed CPM.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        goal (google.ads.googleads.v22.enums.types.FixedCpmGoalEnum.FixedCpmGoal):
            Fixed CPM bidding goal. Determines the exact
            bidding optimization parameters.
        target_frequency_info (google.ads.googleads.v22.common.types.FixedCpmTargetFrequencyGoalInfo):
            Target frequency bidding goal details.

            This field is a member of `oneof`_ ``goal_info``.
    """

    goal: fixed_cpm_goal.FixedCpmGoalEnum.FixedCpmGoal = proto.Field(
        proto.ENUM,
        number=1,
        enum=fixed_cpm_goal.FixedCpmGoalEnum.FixedCpmGoal,
    )
    target_frequency_info: "FixedCpmTargetFrequencyGoalInfo" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="goal_info",
        message="FixedCpmTargetFrequencyGoalInfo",
    )


class FixedCpmTargetFrequencyGoalInfo(proto.Message):
    r"""Target frequency bidding goal details for Fixed CPM bidding
    strategy. Only relevant if the goal of the bidding strategy is
    ``TARGET_FREQUENCY``.

    Attributes:
        target_count (int):
            Target frequency count represents the number
            of times an advertiser wants to show the ad to
            target a single user.
        time_unit (google.ads.googleads.v22.enums.types.FixedCpmTargetFrequencyTimeUnitEnum.FixedCpmTargetFrequencyTimeUnit):
            Time window expressing the period over which you want to
            reach the specified target_count.
    """

    target_count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    time_unit: (
        fixed_cpm_target_frequency_time_unit.FixedCpmTargetFrequencyTimeUnitEnum.FixedCpmTargetFrequencyTimeUnit
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=fixed_cpm_target_frequency_time_unit.FixedCpmTargetFrequencyTimeUnitEnum.FixedCpmTargetFrequencyTimeUnit,
    )


class TargetCpv(proto.Message):
    r"""Target CPV (cost per view) bidding strategy. An automated
    bidding strategy that sets bids to optimize performance given
    the target CPV you set.

    """


class TargetCpc(proto.Message):
    r"""An automated bidding strategy that sets bids to help get as
    many clicks as possible at the target cost-per-click (CPC) you
    set.

    Attributes:
        target_cpc_micros (int):
            Average CPC target.
            This target should be greater than or equal to
            minimum billable unit based on the currency for
            the account.
    """

    target_cpc_micros: int = proto.Field(
        proto.INT64,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import custom_interest_member_type
from google.ads.googleads.v22.enums.types import custom_interest_status
from google.ads.googleads.v22.enums.types import custom_interest_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "CustomInterest",
        "CustomInterestMember",
    },
)


class CustomInterest(proto.Message):
    r"""A custom interest. This is a list of users by interest.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the custom interest. Custom
            interest resource names have the form:

            ``customers/{customer_id}/customInterests/{custom_interest_id}``
        id (int):
            Output only. Id of the custom interest.

            This field is a member of `oneof`_ ``_id``.
        status (google.ads.googleads.v22.enums.types.CustomInterestStatusEnum.CustomInterestStatus):
            Status of this custom interest. Indicates
            whether the custom interest is enabled or
            removed.
        name (str):
            Name of the custom interest. It should be
            unique across the same custom affinity audience.
            This field is required for create operations.

            This field is a member of `oneof`_ ``_name``.
        type_ (google.ads.googleads.v22.enums.types.CustomInterestTypeEnum.CustomInterestType):
            Type of the custom interest, CUSTOM_AFFINITY or
            CUSTOM_INTENT. By default the type is set to
            CUSTOM_AFFINITY.
        description (str):
            Description of this custom interest audience.

            This field is a member of `oneof`_ ``_description``.
        members (MutableSequence[google.ads.googleads.v22.resources.types.CustomInterestMember]):
            List of custom interest members that this
            custom interest is composed of. Members can be
            added during CustomInterest creation. If members
            are presented in UPDATE operation, existing
            members will be overridden.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )
    status: (
        custom_interest_status.CustomInterestStatusEnum.CustomInterestStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=custom_interest_status.CustomInterestStatusEnum.CustomInterestStatus,
    )
    name: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    type_: custom_interest_type.CustomInterestTypeEnum.CustomInterestType = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=custom_interest_type.CustomInterestTypeEnum.CustomInterestType,
        )
    )
    description: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    members: MutableSequence["CustomInterestMember"] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="CustomInterestMember",
    )


class CustomInterestMember(proto.Message):
    r"""A member of custom interest audience. A member can be a
    keyword or url. It is immutable, that is, it can only be created
    or removed but not changed.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        member_type (google.ads.googleads.v22.enums.types.CustomInterestMemberTypeEnum.CustomInterestMemberType):
            The type of custom interest member, KEYWORD
            or URL.
        parameter (str):
            Keyword text when member_type is KEYWORD or URL string when
            member_type is URL.

            This field is a member of `oneof`_ ``_parameter``.
    """

    member_type: (
        custom_interest_member_type.CustomInterestMemberTypeEnum.CustomInterestMemberType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=custom_interest_member_type.CustomInterestMemberTypeEnum.CustomInterestMemberType,
    )
    parameter: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import (
    criterion_category_availability,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "DetailedDemographic",
    },
)


class DetailedDemographic(proto.Message):
    r"""A detailed demographic: a particular interest-based vertical
    to be targeted to reach users based on long-term life facts.

    Attributes:
        resource_name (str):
            Output only. The resource name of the detailed demographic.
            Detailed demographic resource names have the form:

            ``customers/{customer_id}/detailedDemographics/{detailed_demographic_id}``
        id (int):
            Output only. The ID of the detailed
            demographic.
        name (str):
            Output only. The name of the detailed
            demographic. For example,"Highest Level of
            Educational Attainment".
        parent (str):
            Output only. The parent of the detailed_demographic.
        launched_to_all (bool):
            Output only. True if the detailed demographic
            is launched to all channels and locales.
        availabilities (MutableSequence[google.ads.googleads.v22.common.types.CriterionCategoryAvailability]):
            Output only. Availability information of the
            detailed demographic.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    parent: str = proto.Field(
        proto.STRING,
        number=4,
    )
    launched_to_all: bool = proto.Field(
        proto.BOOL,
        number=5,
    )
    availabilities: MutableSequence[
        criterion_category_availability.CriterionCategoryAvailability
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=6,
        message=criterion_category_availability.CriterionCategoryAvailability,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "CriterionCategoryLocaleAvailabilityModeEnum",
    },
)


class CriterionCategoryLocaleAvailabilityModeEnum(proto.Message):
    r"""Describes locale availability mode for a criterion
    availability - whether it's available globally, or a particular
    country with all languages, or a particular language with all
    countries, or a country-language pair.

    """

    class CriterionCategoryLocaleAvailabilityMode(proto.Enum):
        r"""Enum containing the possible
        CriterionCategoryLocaleAvailabilityMode.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ALL_LOCALES (2):
                The category is available to campaigns of all
                locales.
            COUNTRY_AND_ALL_LANGUAGES (3):
                The category is available to campaigns within
                a list of countries, regardless of language.
            LANGUAGE_AND_ALL_COUNTRIES (4):
                The category is available to campaigns within
                a list of languages, regardless of country.
            COUNTRY_AND_LANGUAGE (5):
                The category is available to campaigns within
                a list of country, language pairs.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ALL_LOCALES = 2
        COUNTRY_AND_ALL_LANGUAGES = 3
        LANGUAGE_AND_ALL_COUNTRIES = 4
        COUNTRY_AND_LANGUAGE = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "UserDataErrorEnum",
    },
)


class UserDataErrorEnum(proto.Message):
    r"""Container for enum describing possible user data errors."""

    class UserDataError(proto.Enum):
        r"""Enum describing possible request errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            OPERATIONS_FOR_CUSTOMER_MATCH_NOT_ALLOWED (2):
                Customer is not allowed to perform operations
                related to Customer Match.
            TOO_MANY_USER_IDENTIFIERS (3):
                Maximum number of user identifiers allowed
                for each request is 100 and for each operation
                is 20.
            USER_LIST_NOT_APPLICABLE (4):
                Current user list is not applicable for the
                given customer.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        OPERATIONS_FOR_CUSTOMER_MATCH_NOT_ALLOWED = 2
        TOO_MANY_USER_IDENTIFIERS = 3
        USER_LIST_NOT_APPLICABLE = 4


__all__ = tuple(sorted(__protobuf__.manifest))
